<?php
session_start();
require_once('config.php');
require_once('dbutils.php');
require_once('locationfuncs.php');
require_once('activity.php');

$masterdb = new GenericSqliteTable("var/master.db","master");
$externaldb = new GenericSqliteTable("var/externaldb.db","external");

if(!isset($masterdb[$_GET['id']]))
	die("No such ID");
$record = $masterdb[$_GET['id']];
$nameCol = GetConfigOption('nameCol');
if(!isset($_SESSION['nickname']))
	$_SESSION['nickname'] = "Anonymous";

//*****************
//Captcha
//*****************
$captchaFailed = 0;
$captchaPassed = 0;
if(!isset($_SESSION['captchapassed']))
	$_SESSION['captchapassed'] = array();
if(GetConfigOption('enableCaptcha') and isset($_POST['recaptcha_challenge_field']))
{	
require_once('recaptcha-php-1.11/recaptchalib.php');
$privatekey = GetConfigOption('recaptchaPrivate');;
$resp = recaptcha_check_answer ($privatekey,
                       $_SERVER["REMOTE_ADDR"],
                       $_POST["recaptcha_challenge_field"],
                       $_POST["recaptcha_response_field"]);

if (!$resp->is_valid) {
	  // What happens when the CAPTCHA was entered incorrectly
	  $captchaFailed = 1;
}
else
{
	$captchaPassed = 1;
	$_SESSION['captchapassed'][$_GET['id']] = 1;
}
}
if(!GetConfigOption('enableCaptcha'))
{
	$captchaPassed = 1;
	$_SESSION['captchapassed'][$_GET['id']] = 1;
}


//*****************
//Process User Form
//*****************
$databaseUpdated = 0;
if(isset($_POST['match']) and $_POST['match'] != "NoneOfTheAbove" and !GetConfigOption('readOnly'))
{
	if(!$captchaFailed and $_SESSION['captchapassed'][$_GET['id']])
	{
	//print_r($_POST);
	$newex = array();
	$newex['nickname'] = substr($_POST['nickname'],0,$maxnickname);
	$newex['timestamp'] = time();
	$newex['db'] = "osm";
	list($osmtype, $osmid) = explode("-",$_POST['match']);
	$newex['type'] = substr($osmtype,0,10);
	$newex['id'] = (int)($osmid);
	$newex['license'] = "CC0";
	$newex['karma'] = 1;
	$newex['uuid'] = uniqid();
	$_SESSION['nickname'] = $newex['nickname'];
	
	if(!isset($external['external']))
		$external['external'] = array();
	array_push($external['external'], $newex);
	$externaldb[$_GET['id']] = $external;
	$databaseUpdated = 1;

	AddActivityEvent($record[$nameCol]." Linked",$newex['nickname']." added a database link for ".$record[$nameCol]." on ".htmlspecialchars(date("F j, Y, g:i a",$newex['timestamp'])),$newex['nickname'],"details.php?id=".$_GET['id'],$newex['uuid']);	
	}
	$_SESSION['captchapassed'][$_GET['id']] = 0;
}

$showCandidates = ($captchaPassed and !$databaseUpdated);
if($showCandidates)
{

//*********************************
//Download nearby objects from XAPI
//*********************************

$loc = GetBestLocation($_GET['id']);
if($loc[0] === Null) die("No starting location could be determined.");
//$url = "http://localhost/timscmap/xapi.php/0.6/*[amenity=school]";
$url = GetConfigOption('xapiUrl')."[".GetConfigOption('osmsearchkey')."=".implode("|",explode(";",GetConfigOption('osmsearchvalues')))."]";
$xapilatrange = GetConfigOption('xapilatrange'); 
$xapilonrange = GetConfigOption('xapilonrange');
$url .= "[bbox=".($loc[0][1]-$xapilonrange).",".($loc[0][0]-$xapilatrange).",".($loc[0][1]+$xapilonrange).",".($loc[0][0]+$xapilatrange)."]";
//echo $url;
$ch = curl_init($url);

//curl_setopt($ch, CURLOPT_FILE, $fp);
curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
curl_setopt($ch, CURLOPT_HEADER, 0);

$xapi = curl_exec($ch);
curl_close($ch);

//echo $xapi;
//***************
//Find Candidates
//***************

$xml = new SimpleXMLElement($xapi);
$candidates = array();
$nodepositions = array();
$wayHasNodes = array();

foreach ($xml as $type => $child) {
	//echo $type."\n";
	//print_r( $child->name);
	$name = Null;
	$match = 0;
	$id = (int)$child['id'];
	foreach($child->tag as $t)
	{
		//echo $t['k'].",".$t['v']."\n";
		if($t['k']=="name") $name = $t['v'];
		if($t['k']==GetConfigOption('osmsearchkey') and in_array($t['v'],explode(";",GetConfigOption('osmsearchvalues')))) $match = 1;
	}
	
	if($type=="node")
	{
		$nodepositions[(int)$id] = array((float)$child['lat'],(float)$child['lon']);
	}

	if($type=="way")
	foreach($child->nd as $nd)
	{
		if(!isset($wayHasNodes[(int)$id]))
			$wayHasNodes[(int)$id] = array();
		array_push($wayHasNodes[(int)$id],(int)$nd['ref']);
	}
	if($name===Null) $name = "Unnamed";

	if($match)
	{
		$can = array('name'=>(string)$name, 'type'=>(string)$type, 'id'=> (int)$id);
		if($type=="node")
		{
			$can['lat'] = (float)$child['lat'];
			$can['lon'] = (float)$child['lon'];
		}

		//print_r($can);
		array_push($candidates,$can);
	}
}

//************************
//Calc centre point of way
//************************
//Just use the average node postion, because it is easy to calculate
foreach ($candidates as $i => $can)
{
	if($can['type']!="way") continue;

	$nodes = $wayHasNodes[$can['id']];
	$sumlat = 0.;
	$sumlon = 0.;
	$count = 0;
	foreach($nodes as $n)
	{
		$pos = $nodepositions[$n];
		$sumlat += $pos[0];
		$sumlon += $pos[1];		
		$count++;
	}
	if($count==0) continue;
	$candidates[$i]['lat'] = (float)($sumlat/$count);
	$candidates[$i]['lon'] = (float)($sumlon/$count);
}

//**************
//Calc distances
//**************

$p1 = LatLonToCartesian($loc[0][0],$loc[0][1]);
$canDist = array();
foreach ($candidates as $can)
{
	if(!isset($can['lat']) or !isset($can['lon'])) continue;
	$p2 = LatLonToCartesian($can['lat'],$can['lon']);
	
	$canDist[$can['id']] = PythagDist($p1,$p2);
}
//print_r($canDist);

//*********************
//Sort List by Distance
//*********************

$candDistSortable = array();
foreach($candidates as $can)
{
	$thisCanDist = 10000.;
	if(isset($canDist[$can['id']]))
		$thisCanDist = $canDist[$can['id']];

	array_push($candDistSortable, $thisCanDist);
}
array_multisort($candDistSortable, $candidates);

}

?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<!--
Design by Free CSS Templates
http://www.freecsstemplates.org
Released for free under a Creative Commons Attribution 2.5 License

Name       : Tomatoes
Description: A two-column, fixed-width design for 1024x768 screen resolutions.
Version    : 1.0
Released   : 20110301

-->
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="content-type" content="text/html; charset=utf-8" />
<title><?php echo GetConfigOption('overallTitle').", ".GetConfigOption('subTitle');?></title>
<meta name="keywords" content="" />
<meta name="description" content="" />
<link href="style.css" rel="stylesheet" type="text/css" media="screen" />
</head>
<body>
<div id="wrapper">
	<div id="header">
		<div id="menu">
			<ul>
				<?php $pageCatExplore = 1;
				require('topbar.php'); ?>
			</ul>
		</div>
		<!-- end #menu -->
	</div>
	<div id="logo">
		<h1><a href="#"><?php echo GetConfigOption('overallTitle');?></a></h1>
		<p><em><?php echo GetConfigOption('subTitle');?></em></p>
	</div>
	<hr />
	<!-- end #logo -->
	<div id="page">
		<div id="page-bgtop">
			<div id="content">
<?php 


if($captchaFailed)
{

?>
				<div class="post">
					<h2 class="title"><a href="#">Captcha Failed</a></h2>
					<div class="entry">
						<p>Please try again.</p>
					</div>
				</div>

<?php
}

if(!$captchaPassed and !$databaseUpdated)
{

?>
				<div class="post">
					<h2 class="title"><a href="#">Captcha</a></h2>
					<div class="entry">
						<form action="" method="post"><p>Please complete the following question to see a list of nearby POIs. (This prevents bots from reloading the page and overloading the XAPI server.)</p>
						<p><?php
if(GetConfigOption('enableCaptcha'))
{
	require_once('recaptcha-php-1.11/recaptchalib.php');
	$publickey = GetConfigOption('recaptchaPublic');
	echo recaptcha_get_html($publickey);
}
?>
</p>
						</form>
					</div>
				</div>

<?php
}


if($showCandidates)
{?>
				<div class="post">
					<h2 class="title"><a href="#">Possible OSM Matches for <?php echo $record[$nameCol];?></a></h2>
					<div class="entry">
						<form action="" method="post">
						<p>Your nickname: <input type="text" name="nickname" value="<?php echo htmlspecialchars($_SESSION['nickname']); ?>"/></p>
						<p>
						<?php
						foreach($candidates as $can)
						{
						echo '<input type="radio" name="match" value="'.$can['type'].'-'.$can['id'].'">'.$can['name'];
						if(isset($canDist[$can['id']])) printf( " (%.2fkm)",$canDist[$can['id']]);
						echo " <a href='http://www.openstreetmap.org/browse/".$can['type']."/".$can['id']."'>Browse</a><br/>";
						}
						?>
						<input type="radio" name="match" value="NoneOfTheAbove">None of the above
						</p>
						<p><input type="submit" value="Submit" /> <a href="details?id=<?php echo $_GET['id']; ?>">Cancel</a></p>

						</form>
					</div>
				</div>
<?php
} //End of HTML form

if($databaseUpdated)
{

?>
				<div class="post">
					<h2 class="title"><a href="#">Database Updated</a></h2>
					<div class="entry">
						<p>Thanks for your input. <a href="details.php?id=<?php echo $_GET['id']; ?>">Continue</a>.</p>
					</div>
				</div>

<?php
}
?>
			</div>
			<!-- end #content -->
			<div id="sidebar">
				<ul>
					<?php require('sidebar.php'); ?>

				</ul>
			</div>
			<!-- end #sidebar -->
			<div style="clear: both;">&nbsp;</div>
		</div>
	</div>
	<!-- end #page -->
	<div id="footer">

<?php
	require_once('footer.php');
?>
	</div>
	<!-- end #footer -->
</div>
</body>
</html>
